/*
 * Generation of declaration elements such as parameters and members.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "declaration.h"

/* Generate parameters in each function signature. */

void write_parameters(struct parameter *param, FILE *fp, enum parameter_role role)
{
  if (param == NULL)
    return;

  write_parameter(param, fp, role);
  write_parameter_separator(param->tail, fp);
  write_parameters(param->tail, fp, role);
}

/* Generate a parameter in a signature. */

void write_parameter(struct parameter *param, FILE *fp, enum parameter_role role)
{
  struct identifier *ident = param->identifiers;
  char *type;
  int first = 1;

  while (ident != NULL)
  {
    if (first)
      first = 0;
    else
      fputs(" ", fp);

    /* Emit type identifiers. */

    if (ident->tail != NULL)
      fputs(ident->identifier, fp);

    /* Emit the parameter name. */

    else
    {
      /* Items must have their type identifiers restored. */

      if (param->cls & ITEM_CLASS)
      {
        type = param->cls == FPAGE_ITEM ? L4_FPAGE_TYPE : L4_CAP_TYPE;
        fputs(type, fp);
        fputs(" ", fp);
      }

      fprintf(fp, "%s%s", (role == SIGNATURE_ROLE) && (param->specifier & OUT_PARAMETER) ? "*" : "",
                           ident->identifier);
    }

    ident = ident->tail;
  }
}

void write_parameter_separator(struct parameter *param, FILE *fp)
{
  if (param != NULL)
    fputs(", ", fp);
}

/* Write structure members for word data structures or variable declarations for
   parameters. */

void write_declarations(struct parameter *param, enum specifier direction,
                        enum parameter_class cls, FILE *fp)
{
  for (; param != NULL; param = param->tail)
  {
    if ((param->specifier & direction) && (param->cls & cls))
    {
      fputs("  ", fp);
      write_parameter(param, fp, STRUCTURE_ROLE);
      fputs(";\n", fp);
    }
  }
}
