/*
 * Generation of server dispatch functions.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdlib.h>
#include "common.h"
#include "dispatch.h"
#include "templates.h"



/* Output language, set in the main program. */

extern enum language output_language;



/* Generate a dispatch function signature for the interface. */

void write_dispatcher_signature(FILE *fp, struct interface *iface)
{
  char *type = (output_language == CPP_LANGUAGE) ? iface->name : "void";

  fprintf(fp, "\nvoid dispatch_%s(ipc_message_t *msg, %s *_self);\n", iface->name, type);
}

/* Generate a dispatch function for the different operations. */

void write_dispatcher(struct signature *sig, FILE *fp, struct interface *iface)
{
  char *type = (output_language == CPP_LANGUAGE) ? iface->name : "void";

  fprintf(fp, "\nvoid dispatch_%s(ipc_message_t *msg, %s *_self)\n{\n",
              iface->name, type);

  /* Interpret an operation indicator in the word data if a protocol applies to
     the entire interface. */

  if (get_protocol(iface->attributes))
    fputs("  switch (ipc_message_get_word(msg, 0))\n  {\n", fp);
  else
    fputs("  switch (l4_msgtag_label(msg->tag))\n  {\n", fp);

  write_dispatcher_cases(sig, fp, iface);
  fputs(server_function_dispatcher_body_epilogue, fp);

  fputs("}\n", fp);
}

/* Generate each dispatch possibility within an interface. */

void write_dispatcher_cases(struct signature *sig, FILE *fp, struct interface *iface)
{
  char *opcode, *opname, *protocol;

  if (sig == NULL)
    return;

  opname = get_operation_name(iface, sig);
  opcode = get_opcode_identifier(NULL, opname);

  /* Generate the case and invocation. */

  fprintf(fp, dispatch_function_wrapper_case, opcode, opname);

  /* Generate the other cases. */

  write_dispatcher_cases(sig->tail, fp, iface);

  /* Free allocated strings. */

  free(opcode);
  free(opname);
}
