/*
 * Template output.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

/* Filename definitions. */

#define client_filename                   "%s/%s_client.c"
#define client_header_filename            "%s/%s_client.h"
#define common_filename                   "%s/%s_common.h"
#define compound_interfaces_filename      "%s/%s_interfaces.h"
#define server_filename_c                 "%s/%s_server.c"
#define server_filename_cpp               "%s/%s_server.cc"
#define server_header_filename            "%s/%s_server.h"
#define server_interface_filename         "%s/%s_interface.h"

/* Client templates. */

#define client_prologue \
"#include \"%s_client.h\"\n" \
"#include \"%s_common.h\"\n" \
"#include <ipc/message.h>\n"

#define client_function_body_prologue \
"  ipc_message_t msg;\n" \
"  long err;\n"

#define client_function_body_call \
"\n  ipc_message_request(&msg, %s, _endp);\n\n" \
"  err = l4_error(msg.tag);\n" \
"  if (err)\n" \
"    return err;\n"

/* Compound interface dispatcher templates. */

#define compound_dispatch_prologue \
"#include \"%s_server.h\"\n"

#define compound_handle_function_signature_c \
"\nvoid %s_handle(ipc_message_t *msg, void *_self)"

#define compound_handle_function_signature_cpp \
"\nvoid %s_handle(ipc_message_t *msg, %s *_self)"

#define compound_handle_function \
"{\n" \
"  ipc_message_open(msg);\n" \
"  %s_dispatch(msg, _self);\n" \
"  ipc_message_reply(msg);\n" \
"  ipc_message_discard(msg);\n" \
"}\n"

#define compound_dispatch_function_signature_c \
"\nvoid %s_dispatch(ipc_message_t *msg, void *_self)"

#define compound_dispatch_function_signature_cpp \
"\nvoid %s_dispatch(ipc_message_t *msg, %s *_self)"

#define compound_dispatch_function_prologue \
"{\n" \
"  switch (l4_msgtag_label(msg->tag))\n" \
"  {\n"

#define compound_dispatch_epilogue \
"    default:\n" \
"      ipc_message_send_error(msg, -L4_EBADPROTO);\n" \
"      break;\n" \
"  }\n" \
"}\n"

#define compound_dispatch_header_prologue \
"#pragma once\n\n" \
"#include <ipc/message.h>\n" \
"#include \"%s_interface.h\"\n"

#define compound_dispatch_header_epilogue \
"\n#define %s_expected_items %d\n"

/* Compound interface class definitions. */

#define compound_interface_prologue \
"#pragma once\n\n" \
"#include \"%s_interfaces.h\"\n\n" \
"#ifdef __cplusplus\n\n" \
"class %s :"

#define compound_interface_epilogue \
"\n{\n};\n\n" \
"#endif /* __cplusplus */\n"

#define compound_interfaces_prologue \
"#pragma once\n\n"

/* General header template. */

#define header_prologue \
"#pragma once\n\n" \
"#include <l4/sys/types.h>\n"

/* Server templates. */

#define server_header_prologue \
"#pragma once\n\n" \
"#include <ipc/message.h>\n"

#define server_prologue \
"#include \"%s_server.h\"\n" \
"#include \"%s_common.h\"\n"

#define server_function_body_prologue \
"  long err;\n"

#define server_function_body_call \
"  if (err)\n" \
"    return err;\n"

#define server_function_dispatcher_body_epilogue \
"    default:\n" \
"      ipc_message_send_error(msg, -L4_EBADPROTO);\n" \
"      break;\n" \
"  }\n"

/* Interface class definitions. */

#define server_interface_prologue_cpp \
"\nclass %s\n" \
"{\n" \
"public:"

#define server_interface_epilogue_cpp \
"};\n"

/* Dispatch templates. */

#define dispatch_function_interface_case \
"    case %s:\n" \
"      dispatch_%s(msg, _self);\n" \
"      break;\n\n"

#define dispatch_function_wrapper_case \
"    case %s:\n" \
"      ipc_message_send_error(msg, wrap_%s(msg, _self));\n" \
"      break;\n\n"
